from kapybara import WebDriverWait, By, EC as ec
from kapybara.shortcuts import TimeoutException
import time
from datetime import datetime
from .is_drug_name_similar import is_drug_name_similar
from .extract_med_data import extract_medication_data
import re

def introducing_history_medication(
    driver, drug_counter, drug_names_set, clicked_drugs, start_date, end_date
):
    # 点击加载门诊用药
    element = WebDriverWait(driver, 10).until(
        ec.presence_of_element_located(
            (By.XPATH, '//div[contains(text(), "加载门诊用药")]')
        )
    )
    driver.execute_script('arguments[0].click();', element)
    time.sleep(3.5)

    all_outpatient_meds = []  # 用于存储所有门诊用药记录
    try:
        # 等待所有匹配的元素出现
        yp = WebDriverWait(driver, 8).until(
            ec.presence_of_all_elements_located(
                (
                    By.XPATH,
                    "//div[contains(@class, 'x-grid-group ')]/div[2]/div/table/tbody/tr",
                )
            )
        )
        yp_number = len(yp)
        all_outpatient_meds = extract_medication_data(yp)  # 提取数据
    except TimeoutException:
        yp_number = 0

    if yp_number == 0:
        print('门诊用药无用药需要引入')
        element = WebDriverWait(driver, 10).until(
            ec.presence_of_element_located((By.XPATH, "//button[text()='选择']"))
        )
        driver.execute_script('arguments[0].click();', element)
        return clicked_drugs, all_outpatient_meds
    else:
        print('开始引入用药')
        for row in yp:
            # 如果已经引入了五个药品，停止引入
            if drug_counter >= 5:
                print('最多引入五个药品，停止引入')
                break

            drug_name = row.find_element(By.XPATH, './td[3]/div').text

            drug_name = drug_name.replace('（', '(').replace('）', ')')
            # drug_name = re.sub(r'\(.*?\)', '', drug_name).strip()

            # 提取用药日期
            drug_date_text = row.find_element(By.XPATH, './td[6]/div').text

            if drug_date_text == '':
                continue

            drug_date = datetime.strptime(drug_date_text, '%Y-%m-%d')
            # 判断随访日期是否在指定范围内
            if start_date <= drug_date <= end_date:
                pass
            else:
                continue

            if is_drug_name_similar(drug_name, clicked_drugs):
                continue
            if drug_name in drug_names_set and drug_name not in clicked_drugs:
                print('正在引入:', drug_name)

                # 点击药物
                try:
                    drug_element = WebDriverWait(driver, 5).until(
                        ec.presence_of_element_located(
                            (By.XPATH, f'//div[contains(text(), "{drug_name}")]')
                        )
                    )
                except:
                    try:
                        drug_name = drug_name.replace('(', '（').replace(')', '）')
                        drug_element = WebDriverWait(driver, 5).until(
                            ec.presence_of_element_located(
                                (By.XPATH, f'//div[contains(text(), "{drug_name}")]')
                            )
                        )
                    except:
                        drug_name = drug_name.replace('（', '(').replace('）', ')')
                        drug_name = re.sub(r'\(.*?\)', '', drug_name).strip()

                        drug_element = WebDriverWait(driver, 20).until(
                            ec.presence_of_element_located(
                                (By.XPATH, f'//div[contains(text(), "{drug_name}")]')
                            )
                        )

                # 滚动到药物元素
                driver.execute_script(
                    'arguments[0].scrollIntoView(true);', drug_element
                )
                time.sleep(1)  # 确保滚动操作完成并页面稳定

                # 尝试模拟鼠标事件
                driver.execute_script(
                    "var evt = document.createEvent('MouseEvents');"
                    "evt.initMouseEvent('mousedown', true, true, window);"
                    'arguments[0].dispatchEvent(evt);'
                    "evt.initMouseEvent('mouseup', true, true, window);"
                    'arguments[0].dispatchEvent(evt);'
                    "evt.initMouseEvent('click', true, true, window);"
                    'arguments[0].dispatchEvent(evt);',
                    drug_element,
                )
                time.sleep(1.5)
                clicked_drugs.add(drug_name)  # 将点击过的药品添加到集合中
                drug_counter += 1

        # 点击选择按钮
        element = WebDriverWait(driver, 10).until(
            ec.presence_of_element_located((By.XPATH, "//button[text()='选择']"))
        )
        driver.execute_script('arguments[0].click();', element)
        time.sleep(1.5)
    return clicked_drugs, all_outpatient_meds




from pathlib import Path
import csv

# 新增：CSV文件路径
OUTPUT_FILE = Path('./执行结果/用药记录.csv')


def save_medication_records(sfzh, history_meds, outpatient_meds):
    """将收集到的用药记录保存到CSV文件"""
    try:
        OUTPUT_FILE.parent.mkdir(parents=True, exist_ok=True)
        history_str = '; '.join([f'{m["name"]} ({m["date"]})' for m in history_meds])
        outpatient_str = '; '.join(
            [f'{m["name"]} ({m["date"]})' for m in outpatient_meds]
        )
        file_exists = OUTPUT_FILE.exists()
        with open(OUTPUT_FILE, 'a', newline='', encoding='utf-8-sig') as f:
            writer = csv.writer(f)
            if not file_exists:
                writer.writerow(['身份证号', '历史用药', '门诊用药'])
            writer.writerow([sfzh, history_str, outpatient_str])
        print(f'用药记录已保存到: {OUTPUT_FILE}')
    except Exception as e:
        print(f'保存用药记录失败: {e}')



from kapybara.shared_data import shared_data
from kapybara.shortcuts import TimeoutException, WebDriverWait, By, EC as ec
from kapybara import FormElement
import time

def alert_element_exist(xpath: str) -> bool:
    driver = shared_data.driver
    try:
        element = WebDriverWait(driver, 3).until(
            ec.visibility_of_element_located((By.XPATH, xpath))
        )
    except TimeoutException:
        return False
    else:
        return True


def handle_post_save_popups(driver, yes_option):
    """
    统一处理点击“保存”后可能出现的各种弹窗。
    返回一个元组 (status, message)，其中 status 表示操作是否应该继续。
    """
    try:
        # 检查“药品名称不能为空”的弹窗
        if alert_element_exist('//span[contains(text(), "药品名称不能为空或无")]'):
            print('错误：药品列表为空，无法保存。')
            driver.find_element(By.XPATH, "//button[text()='确定']").click()
            return (False, 'empty_drug_list')

        # 检查“本季度已做过慢病随访”的弹窗
        if alert_element_exist(f"//button[text()='{yes_option}']"):
            print(f"检测到重复随访弹窗，选择：'{yes_option}'")
            FormElement(f"//button[text()='{yes_option}']").click()
            if yes_option == '否':
                return (False, 'duplicate_followup_declined')
            # 如果选择了“是”，则可能还有后续弹窗，继续检查
            time.sleep(1)  # 等待下一个弹窗出现

        # 检查“需要先保存随访”的弹窗
        if alert_element_exist('//span[contains(text(), "需要先")]'):
            print('需要先保存随访，正在确认...')
            driver.find_element(By.XPATH, '//button[contains(text(), "是")]').click()
            time.sleep(1)
            # 点击确认后的“确定”按钮
            driver.find_element(By.XPATH, "//button[text()='确定']").click()
            print('用药情况已保存。')
            # 检查并关闭“是否加入到个人服务计划中”的弹窗
            time.sleep(0.5)
            if alert_element_exist(
                '//span[contains(text(), "是否加入到个人服务计划中")]'
            ):
                driver.find_element(
                    By.XPATH, '//button[contains(text(), "否")]'
                ).click()
            return (True, 'saved_via_nested_dialog')

        # 检查通用的“确定”按钮，表示成功保存
        confirm_button = driver.find_elements(By.XPATH, "//button[text()='确定']")
        if confirm_button:
            print('用药情况已保存。')
            confirm_button[0].click()
            return (True, 'saved_successfully')

    except Exception as e:
        # 捕获意外错误，例如元素在检查后立即消失
        print(f'处理弹窗时发生未知错误: {e}')
        return (True, 'unknown_error')  # 假设操作可继续，避免卡死

    return (True, 'no_popup_found')  # 没有找到任何弹窗




import difflib
from yxmb_compatlib.config import load_config

# 加载配置
config = load_config()
# 从配置中获取药品相似度阈值，如果未找到则使用默认值 0.8
MEDICINE_THRESHOLD = config.get('medicine', {}).get('name_similarity_threshold', 0.8)


def is_drug_name_similar(drug_name, drug_names_set, threshold=None):
    """
    判断一个药物名称是否与集合中的任何药物名称相似。

    :param drug_name: 当前待检查的药物名称
    :param drug_names_set: 已知药物名称的集合
    :param threshold: 相似度阈值，如果为 None，则使用配置文件中的值
    :return: 如果相似的药物存在，则返回True，否则返回False
    """
    # 如果未提供阈值，则使用从配置中加载的值
    if threshold is None:
        threshold = MEDICINE_THRESHOLD

    for name in drug_names_set:
        # 计算相似度
        similarity = difflib.SequenceMatcher(None, drug_name, name).ratio()

        # 如果相似度达到阈值，返回True
        if similarity >= threshold:
            return True

    # 如果没有找到相似药物，返回False
    return False



from kapybara import WebDriverWait, By, EC as ec
from kapybara.shortcuts import TimeoutException, StaleElementReferenceException
import time

def delete_existing_meds_func(driver):
    # 删除原有的用药记录
    try:
        # 获取所有用药记录的容器
        child_divs = WebDriverWait(driver, 15).until(
            ec.visibility_of_all_elements_located((By.XPATH, '//*[@id="ext-gen21"]/div'))
        )

        # 检查是否有用药记录
        if not child_divs:
            print("无用药记录")
            return

        # 获取药物名称元素，用于判断列位置
        drug_name = WebDriverWait(driver, 10).until(
            ec.presence_of_element_located((By.XPATH,
                                            '/html/body/form/div[2]/div/div[3]/table/tbody/tr/td/div/div/div/div/div/div[1]/div[2]/div/div[1]/table/tbody/tr/td[1]/div'))
        )
        drug_name_value = drug_name.text

        # 从后往前删除（避免索引变化问题）
        for i in range(len(child_divs), 0, -1):
            try:
                # 根据药物名称是否为空确定删除按钮的位置
                if len(drug_name_value) == 0:
                    delete_xpath = f'/html/body/form/div[2]/div/div[3]/table/tbody/tr/td/div/div/div/div/div/div[1]/div[2]/div/div[{i}]/table/tbody/tr/td[6]/div/img'
                else:
                    delete_xpath = f'/html/body/form/div[2]/div/div[3]/table/tbody/tr/td/div/div/div/div/div/div[1]/div[2]/div/div[{i}]/table/tbody/tr/td[5]/div/img'

                # 定位删除按钮
                delete_button = WebDriverWait(driver, 10).until(
                    ec.element_to_be_clickable((By.XPATH, delete_xpath)))

                # 滚动到元素并确保在视图中
                driver.execute_script("""
                    arguments[0].scrollIntoView({
                        behavior: 'auto', 
                        block: 'center',
                        inline: 'center'
                    });
                """, delete_button)

                # 添加短暂等待确保元素稳定
                time.sleep(0.3)

                # 使用JavaScript点击删除按钮（避免遮挡问题）
                # driver.execute_script("arguments[0].click();", delete_button)
                delete_button.click()

                # 处理可能的确认对话框
                try:
                    confirm_btn = WebDriverWait(driver, 3).until(
                        ec.element_to_be_clickable(
                            (By.XPATH, "//button[text()='是']"))
                    )
                    confirm_btn.click()
                except TimeoutException:
                    pass  # 没有确认对话框

                time.sleep(0.5)

                # 等待列表刷新
                WebDriverWait(driver, 3).until(
                    lambda d: len(d.find_elements(By.XPATH, '//*[@id="ext-gen21"]/div')) < len(child_divs)
                )

                # 更新child_divs计数
                child_divs = driver.find_elements(By.XPATH, '//*[@id="ext-gen21"]/div')

            except StaleElementReferenceException:
                print(f"第 {i} 条记录状态已更新，继续操作...")
                # 刷新元素列表
                child_divs = driver.find_elements(By.XPATH, '//*[@id="ext-gen21"]/div')
                continue

            except TimeoutException:
                print(f"第 {i} 条记录删除超时，尝试继续")
                continue

            except Exception as e:
                print(f"删除第 {i} 条记录时出错: {str(e)}")
                driver.save_screenshot(f"delete_error_{i}.png")
                continue
        print("所有用药记录已成功删除")
    except TimeoutException:
        print("没有找到用药记录容器")
    except Exception as e:
        print(f"删除用药记录时发生错误: {str(e)}")



from kapybara import By

def extract_medication_data(rows):
    """从用药记录行中提取药品数据"""
    medications = []
    for row in rows:
        try:
            drug_name = row.find_element(By.XPATH, './td[3]/div').text
            drug_date = row.find_element(By.XPATH, './td[6]/div').text
            drug_name = drug_name.replace('（', '(').replace('）', ')').strip()
            if drug_name:
                medications.append({'name': drug_name, 'date': drug_date})
        except Exception:
            # 忽略无法提取的行
            pass
    return medications
